@@ -4,6 +4,7 @@
 #                   for usage in the ubuntu-dev-tools package
 #
 #   Copyright © 2009-2010 Michael Bienia <geser@ubuntu.com>
+#               2011      Stefano Rivera <stefanor@ubuntu.com>
 #
 #   This program is free software; you can redistribute it and/or
 #   modify it under the terms of the GNU General Public License
@@ -25,7 +26,9 @@
 #httplib2.debuglevel = 1
 
 import sys
+import urllib2
 
+from debian.changelog import Changelog, Version
 from launchpadlib.launchpad import Launchpad as LP
 from launchpadlib.errors import HTTPError
 from lazr.restfulclient.resource import Entry
@@ -349,6 +352,8 @@ class SourcePackagePublishingHistory(BaseWrapper):
     resource_type = 'source_package_publishing_history'
 
     def __init__(self, *args):
+        self._changelog = None
+        self._binaries = None
         # Don't share _builds between different
         # SourcePackagePublishingHistory objects
         if '_builds' not in self.__dict__:
@@ -372,6 +377,49 @@ class SourcePackagePublishingHistory(BaseWrapper):
         '''
         return self._lpobject.component_name
 
+    def getChangelog(self, since_version=None):
+        '''
+        Return the changelog, optionally since a particular version
+        May return None if the changelog isn't available
+        Only available in the devel API, not 1.0
+        '''
+        if self._changelog is None:
+            url = self._lpobject.changelogUrl()
+            if url is None:
+                print >> sys.stderr, ('E: No changelog available for %s %s',
+                                      (self.getPackageName(),
+                                       self.getVersion()))
+                return None
+
+            try:
+                self._changelog = urllib2.urlopen(url).read()
+            except urllib2.HTTPError, error:
+                print >> sys.stderr, ('%s: %s' % (url, error))
+                return None
+
+        if since_version is None:
+            return self._changelog
+
+        if isinstance(since_version, basestring):
+            since_version = Version(since_version)
+
+        new_entries = []
+        for block in Changelog(self._changelog):
+            if block.version <= since_version:
+                break
+            new_entries.append(unicode(block))
+        return u''.join(new_entries)
+
+    def getBinaries(self):
+        '''
+        Returns the resulting BinaryPackagePublishingHistorys
+        '''
+        if self._binaries is None:
+            self._binaries = [BinaryPackagePublishingHistory(bpph)
+                              for bpph in
+                              self._lpobject.getPublishedBinaries()]
+        return self._binaries
+
     def _fetch_builds(self):
         '''Populate self._builds with the build records.'''
         builds = self.getBuilds()
@@ -424,6 +472,31 @@ class SourcePackagePublishingHistory(BaseWrapper):
             self.getPackageName(), '\n'.join(res))
 
 
+class BinaryPackagePublishingHistory(BaseWrapper):
+    '''
+    Wrapper class around a LP binary package object.
+    '''
+    resource_type = 'binary_package_publishing_history'
+
+    def getPackageName(self):
+        '''
+        Returns the binary package name.
+        '''
+        return self._lpobject.binary_package_name
+
+    def getVersion(self):
+        '''
+        Returns the version of the binary package.
+        '''
+        return self._lpobject.binary_package_version
+
+    def getComponent(self):
+        '''
+        Returns the component of the binary package.
+        '''
+        return self._lpobject.component_name
+
+
 class MetaPersonTeam(MetaWrapper):
     @property
     def me(cls):
