@@ -20,12 +20,17 @@
 #   Please see the /usr/share/common-licenses/GPL-2 file for the full text
 #   of the GNU General Public License license.
 
-from ubuntutools.distro_info import DebianDistroInfo
-from ubuntutools.requestsync.common import raw_input_exit_on_ctrlc
+import re
+import urllib2
+
+from debian.deb822 import Changes
+from distro_info import DebianDistroInfo
+
 from ubuntutools.lp.lpapicache import (Launchpad, Distribution, PersonTeam,
                                        DistributionSourcePackage)
+from ubuntutools.question import confirmation_prompt
 
-def getDebianSrcPkg(name, release):
+def get_debian_srcpkg(name, release):
     debian = Distribution('debian')
     debian_archive = debian.getArchive()
 
@@ -33,13 +38,13 @@ def getDebianSrcPkg(name, release):
 
     return debian_archive.getSourcePackage(name, release)
 
-def getUbuntuSrcPkg(name, release):
+def get_ubuntu_srcpkg(name, release, pocket='Release'):
     ubuntu = Distribution('ubuntu')
     ubuntu_archive = ubuntu.getArchive()
 
-    return ubuntu_archive.getSourcePackage(name, release)
+    return ubuntu_archive.getSourcePackage(name, release, pocket)
 
-def needSponsorship(name, component, release):
+def need_sponsorship(name, component, release):
     '''
     Check if the user has upload permissions for either the package
     itself or the component
@@ -55,11 +60,11 @@ Your sync request shall require an approval by a member of the appropriate
 sponsorship team, who shall be subscribed to this bug report.
 This must be done before it can be processed by a member of the Ubuntu Archive
 team.'''
-        raw_input_exit_on_ctrlc('If the above is correct please press [Enter] ')
+        confirmation_prompt()
 
     return need_sponsor
 
-def checkExistingReports(srcpkg):
+def check_existing_reports(srcpkg):
     '''
     Check existing bug reports on Launchpad for a possible sync request.
 
@@ -67,11 +72,14 @@ def checkExistingReports(srcpkg):
     '''
 
     # Fetch the package's bug list from Launchpad
-    pkg = Distribution('ubuntu').getSourcePackage(name = srcpkg)
-    pkgBugList = pkg.getBugTasks()
+    pkg = Distribution('ubuntu').getSourcePackage(name=srcpkg)
+    pkg_bug_list = pkg.searchTasks(status=["Incomplete", "New", "Confirmed",
+                                           "Triaged", "In Progress",
+                                           "Fix Committed"],
+                                   omit_duplicates=True)
 
     # Search bug list for other sync requests.
-    for bug in pkgBugList:
+    for bug in pkg_bug_list:
         # check for Sync or sync and the package name
         if not bug.is_complete and 'ync %s' % srcpkg in bug.title:
             print ('The following bug could be a possible duplicate sync bug '
@@ -80,17 +88,51 @@ def checkExistingReports(srcpkg):
                    'Please check the above URL to verify this before '
                    'continuing.'
                    % (bug.title, bug.web_link))
-            raw_input_exit_on_ctrlc('Press [Enter] to continue or [Ctrl-C] '
-                                    'to abort. ')
+            confirmation_prompt()
 
-def postBug(srcpkg, subscribe, status, bugtitle, bugtext):
+def get_ubuntu_delta_changelog(srcpkg):
+    '''
+    Download the Ubuntu changelog and extract the entries since the last sync
+    from Debian.
+    '''
+    archive = Distribution('ubuntu').getArchive()
+    spph = archive.getPublishedSources(source_name=srcpkg.getPackageName(),
+                                       exact_match=True, pocket='Release')
+    debian_info = DebianDistroInfo()
+    topline = re.compile(r'^(\w%(name_chars)s*) \(([^\(\) \t]+)\)'
+                         r'((\s+%(name_chars)s+)+)\;'
+                         % {'name_chars': '[-+0-9a-z.]'},
+                         re.IGNORECASE)
+    delta = []
+    for record in spph:
+        changes_url = record.changesFileUrl()
+        if changes_url is None:
+            # Native sync
+            break
+        changes = Changes(urllib2.urlopen(changes_url))
+        for line in changes['Changes'].splitlines():
+            line = line[1:]
+            m = topline.match(line)
+            if m:
+                distribution = m.group(3).split()[0].split('-')[0]
+                if debian_info.valid(distribution):
+                    break
+            if line.startswith(u'  '):
+                delta.append(line)
+        else:
+            continue
+        break
+
+    return '\n'.join(delta)
+
+def post_bug(srcpkg, subscribe, status, bugtitle, bugtext):
     '''
     Use the LP API to file the sync request.
     '''
 
     print ('The final report is:\nSummary: %s\nDescription:\n%s\n'
            % (bugtitle, bugtext))
-    raw_input_exit_on_ctrlc('Press [Enter] to continue or [Ctrl-C] to abort. ')
+    confirmation_prompt()
 
     if srcpkg:
         bug_target = DistributionSourcePackage(
