@@ -25,13 +25,12 @@ import sys
 import smtplib
 import socket
 
-from debian.changelog import Version
+from debian.changelog import Changelog, Version
 from devscripts.logger import Logger
 from distro_info import DebianDistroInfo
 
 from ubuntutools.archive import rmadison, FakeSPPH
-from ubuntutools.requestsync.common import (get_changelog,
-                                            raw_input_exit_on_ctrlc)
+from ubuntutools.question import confirmation_prompt, YesNoQuestion
 from ubuntutools import subprocess
 from ubuntutools.lp.udtexceptions import PackageNotFoundException
 
@@ -57,7 +56,7 @@ def _get_srcpkg(distro, name, release):
                                        % (name, distro.capitalize(), release))
     pkg = max(lines, key=lambda x: Version(x['version']))
 
-    return FakeSPPH(pkg['source'], pkg['version'], pkg['component'])
+    return FakeSPPH(pkg['source'], pkg['version'], pkg['component'], distro)
 
 def get_debian_srcpkg(name, release):
     return _get_srcpkg('debian', name, release)
@@ -71,17 +70,12 @@ def need_sponsorship(name, component, release):
     component.
     '''
 
-    while True:
-        print ("Do you have upload permissions for the '%s' component "
-               "or the package '%s' in Ubuntu %s?"
-               % (component, name, release))
-        val = raw_input_exit_on_ctrlc("If in doubt answer 'n'. [y/N]? ")
-        if val.lower() in ('y', 'yes'):
-            return False
-        elif val.lower() in ('n', 'no', ''):
-            return True
-        else:
-            print 'Invalid answer'
+    val = YesNoQuestion().ask("Do you have upload permissions for the "
+                              "'%s' component or the package '%s' in "
+                              "Ubuntu %s?\n"
+                              "If in doubt answer 'n'."
+                               % (component, name, release), 'no')
+    return val == 'no'
 
 def check_existing_reports(srcpkg):
     '''
@@ -90,14 +84,14 @@ def check_existing_reports(srcpkg):
     print ('Please check on '
            'https://bugs.launchpad.net/ubuntu/+source/%s/+bugs\n'
            'for duplicate sync requests before continuing.' % srcpkg)
-    raw_input_exit_on_ctrlc('Press [Enter] to continue or [Ctrl-C] to abort. ')
+    confirmation_prompt()
 
 def get_ubuntu_delta_changelog(srcpkg):
     '''
     Download the Ubuntu changelog and extract the entries since the last sync
     from Debian.
     '''
-    changelog = get_changelog(srcpkg, 'ubuntu')
+    changelog = Changelog(srcpkg.getChangelog())
     if changelog is None:
         return u''
     delta = []
@@ -166,7 +160,7 @@ Content-Type: text/plain; charset=UTF-8
 %s''' % (myemailaddr, to, bugtitle, signed_report)
 
     print 'The final report is:\n%s' % mail
-    raw_input_exit_on_ctrlc('Press [Enter] to continue or [Ctrl-C] to abort. ')
+    confirmation_prompt()
 
     # connect to the server
     try:
